/*
 * WsIngest.h  Mar 29, 1996 [fc]
 * ****************************************
 */

/*
 * Some defaults
 */
#define DF_SAMPLE_INTERVAL 300		/* sample interval */
#define BADVAL -99.9                    /* bad data flag */
#define MIH2MS 0.447			/* miles/hour to meters/sec */
#define MS2MIH 2.237			/* meters/sec to miles/hour */
#define MIH2KMH 1.609			/* mph -> km/h */
#define KMH2MIH 0.622			/* km/h -> mph */
#define IHG2MBC 0.3386			/* inches hg to millibars / 100 */
#define IN2MMC 0.254			/* inches to mm / 100 */
#define MB2IN 0.0295 			/* mb to inches */
#define MM2IN 0.03937			/* mm to inches */
#define IN2MM 25.4			/* inches to mm */
#define KPH2MPS 0.2778			/* Km/Hr to Mtrs/Sec */
#define FIVENINTHS 0.5555		/* conversion for F to C */
#define NINEFIFTHS 1.8			/* conversion for C to F */
#define DEGTORAD (2.0 * M_PI) / 360.0	/* degrees to radians */
#define DF_ST_LOCATION "Anyplace, USA"	/* default location */
#define DF_ST_MODEL "Generic Station"	/* default model */
#define DF_LATITUDE 40.0		/* default latitude */
#define DF_LONGITUDE 105.0		/* default longitude */
#define DF_ELEVATION 1600.0		/* default elevation */
#define DER_CPRES 1			/* calibrated pressure bit */
#define DER_DPDT  2			/* calibrated dpdt bit */
#define DER_DEWP  4			/* calibrated deopoint bit */
#define DER_WCHIL 8			/* calibrated wind chill bit */

/*
 * A structure to hold running averages.
 */
typedef struct Averager
{
	int a_NSamples;			/* How many samples we have */
	float a_Sum;			/* The running sum	    */
} Averager;


/*
 * Structure to hold everything coming from a "generic" weather station
 */

struct ws_data
{
	float latitude, longitude, elevation;
	char location[100];
	char model[100];

	float outtemp, outhum, wchill, dp, pres, cpres, dpdt;
	float wspd, wdir, rain, intemp, inhum;
	float peak_gust, calc_av_wdir;

	Averager av_outtemp, av_outhum, av_wchill;
	Averager av_dp, av_pres, av_cpres, av_dpdt;
	Averager av_wspd, av_intemp, av_inhum;
	Averager av_uwind, av_vwind;

	time_t snap_time, samp_time;
	char wdirname[4];

	/* calibrations additives */
	float a_outtemp, a_outhum, a_pres;
	float a_wdir, a_intemp, a_inhum;

	/* calibration multipliers */
	float m_outtemp, m_outhum, m_pres;
	float m_wspd, m_rain, m_intemp, m_inhum;
#if 0
	float a_wchill, a_dp;
	float m_wchill, m_dp;
#endif
};


/*
 * Function Prototypes
 */

void gmt_log (char *str);
int gmt_minute ();

#ifdef SOLARIS
void dummy_hndl ();
void sol_usleep (int usec);
#endif

/*
 * Averager functions.
 */
void av_Zero (Averager *);
void av_Accumulate (float, Averager *);
float av_GetAverage (Averager *);

void calibrate_ws (struct ws_data *ws);
void wsd_av_zero (struct ws_data *wsd);
void wsd_av_sum (struct ws_data *wsd);
void wsd_average (struct ws_data *wsd);
void wsd_av_dump (struct ws_data *ws);
void wsd_fingfile(struct ws_data *ws, char *dir);
void wsd_datafile(struct ws_data *ws, char *dir);
void calc_derived (struct ws_data *ws, unsigned int fields);
void wsd_dump (struct ws_data *wsd);
void wsv_dump (struct ws_data *wsd);

void read_config_location (struct ws_data *wm);
void dump_config_location (struct ws_data *wm);
void read_config_cals (struct ws_data *wm);
void dump_config_cals (struct ws_data *wm);
void hex_dump (unsigned char *str, int len);
void str_hex_dump (char *rstr, char *lbl, unsigned char *str, int len);

